import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
var _jsxFileName = "C:\\Dev\\20_gsoft\\sg-orbit\\packages\\react-components\\components\\date-range-picker\\src\\date-range-picker-calendar.jsx";
import _JSXStyle from "styled-jsx/style";
import React from "react";
import { DayPickerRangeController } from "react-dates";
import { PRESET_SHAPE } from "./presets";
import { PureComponent, cloneElement } from "react";
import { START_DATE } from "react-dates/constants";
import { arrayOf, bool, func, node, object, shape, string } from "prop-types";
import { isNil } from "lodash";
import moment from "moment";
const PHRASES = {
  chooseAvailableStartDate: (_ref) => {
    let {
      date
    } = _ref;
    return "Choose ".concat(date, ".");
  },
  chooseAvailableEndDate: (_ref2) => {
    let {
      date
    } = _ref2;
    return "Choose ".concat(date, ".");
  }
};
export class DateRangePickerCalendar extends PureComponent {
  constructor() {
    super(...arguments);

    _defineProperty(this, "state", {
      // Must be non-null in order to select dates.
      focusedInput: START_DATE
    });

    _defineProperty(this, "handleFocusChange", focusedInput => {
      this.setState({
        focusedInput
      });
    });

    _defineProperty(this, "handleDatesChange", (_ref3) => {
      let {
        startDate,
        endDate
      } = _ref3;
      const {
        onDatesChange
      } = this.props;
      const {
        focusedInput
      } = this.state;

      if (focusedInput === START_DATE) {
        if (!isNil(startDate) && !isNil(endDate)) {
          // By default, when the user select a valid full range then select a date previous to the range, react-dates will extend the current range instead of starting a new one.
          // It works this way because react-dates doesn't reset the endDate.
          onDatesChange(startDate, null, null, this.props);
        } else {
          onDatesChange(startDate, endDate, null, this.props);
        }
      } else {
        // Enable selection of a new single date or range when an end date is selected.
        // This is mostly usefull to allow the selection of a single date after a range has been selected.
        // The default behavior is to select a new end date for the current range.
        if (!isNil(endDate)) {
          this.resetFocusedInput();
        }

        onDatesChange(startDate, endDate, null, this.props);
      }
    });

    _defineProperty(this, "handleClear", () => {
      const {
        onDatesChange
      } = this.props;
      this.resetFocusedInput();
      onDatesChange(null, null, null, this.props);
    });

    _defineProperty(this, "handleApply", event => {
      const {
        onApply
      } = this.props;
      this.resetFocusedInput();
      onApply(event, this.props);
    });

    _defineProperty(this, "handleSelectPreset", (event, preset) => {
      const {
        onDatesChange
      } = this.props;
      this.resetFocusedInput();
      onDatesChange(preset.startDate, preset.endDate, preset.text, this.props);
    });

    _defineProperty(this, "isDayBlocked", day => {
      const {
        minDate,
        maxDate
      } = this.props;

      if (!isNil(minDate) && !isNil(maxDate)) {
        return day.isBefore(minDate, "day") || day.isAfter(maxDate, "day");
      }

      if (!isNil(minDate)) {
        return day.isBefore(minDate, "day");
      }

      if (!isNil(maxDate)) {
        return day.isAfter(maxDate, "day");
      }

      return false;
    });

    _defineProperty(this, "isMonthBlocked", month => {
      const {
        minDate,
        maxDate
      } = this.props;

      if (!isNil(minDate) || !isNil(maxDate)) {
        const firstDay = moment(month).startOf("month");
        const lastDay = moment(month).endOf("month");
        return this.isDayBlocked(firstDay) && this.isDayBlocked(lastDay);
      }

      return false;
    });

    _defineProperty(this, "getInitialVisibleMonth", () => {
      const {
        startDate,
        endDate
      } = this.props;
      const initialMonth = startDate || endDate || moment();
      const nextMonth = moment(initialMonth).add(1, "months");

      if (this.isMonthBlocked(nextMonth)) {
        // When the next month is blocked, show the previous and current months instead of the current and next months.
        return moment(initialMonth).subtract(1, "months");
      }

      return initialMonth;
    });
  }

  resetFocusedInput() {
    this.setState({
      focusedInput: START_DATE
    });
  }

  getCssClasses() {
    const {
      className
    } = this.props;
    const defaultClasses = "calendar flex mt3";
    return isNil(className) ? defaultClasses : "".concat(defaultClasses, " ").concat(className);
  }

  renderPresets() {
    const {
      startDate,
      endDate,
      minDate,
      maxDate,
      presetsComponent,
      presets,
      presetsIcon
    } = this.props;
    return cloneElement(presetsComponent, {
      startDate,
      endDate,
      minDate,
      maxDate,
      onSelectPreset: this.handleSelectPreset,
      presets,
      icon: presetsIcon
    });
  }

  renderNavPrev() {
    const {
      navPrevIcon
    } = this.props;
    return React.createElement("div", {
      tabIndex: "0",
      className: "flex",
      __source: {
        fileName: _jsxFileName,
        lineNumber: 162
      },
      __self: this
    }, navPrevIcon);
  }

  renderNavNext() {
    const {
      navNextIcon
    } = this.props;
    return React.createElement("div", {
      tabIndex: "0",
      className: "flex",
      __source: {
        fileName: _jsxFileName,
        lineNumber: 168
      },
      __self: this
    }, navNextIcon);
  }

  renderButtons() {
    const {
      startDate,
      endDate,
      allowSingleDateSelection,
      buttons,
      clearText,
      applyText
    } = this.props;
    return cloneElement(buttons, {
      startDate,
      endDate,
      onClear: this.handleClear,
      onApply: this.handleApply,
      allowSingleDateSelection,
      clearText,
      applyText
    });
  }

  render() {
    const {
      startDate,
      endDate,
      allowSingleDateSelection,
      minDate,
      maxDate
    } = this.props;
    const {
      focusedInput
    } = this.state;
    return React.createElement("div", {
      className: "jsx-1959698239" + " " + (this.getCssClasses() || ""),
      __source: {
        fileName: _jsxFileName,
        lineNumber: 190
      },
      __self: this
    }, this.renderPresets(), React.createElement("div", {
      className: "jsx-1959698239" + " " + "flex flex-column",
      __source: {
        fileName: _jsxFileName,
        lineNumber: 192
      },
      __self: this
    }, React.createElement(DayPickerRangeController, {
      startDate: startDate,
      endDate: endDate,
      onDatesChange: this.handleDatesChange,
      onFocusChange: this.handleFocusChange,
      focusedInput: focusedInput,
      minimumNights: allowSingleDateSelection ? 0 : 1,
      minDate: minDate,
      maxDate: maxDate,
      navPrev: this.renderNavPrev(),
      navNext: this.renderNavNext(),
      isDayBlocked: !isNil(minDate) || !isNil(maxDate) ? this.isDayBlocked : undefined,
      initialVisibleMonth: this.getInitialVisibleMonth,
      numberOfMonths: 2,
      phrases: PHRASES,
      transitionDuration: 0,
      noBorder: true,
      keepOpenOnDateSelect: true,
      hideKeyboardShortcutsPanel: true,
      __source: {
        fileName: _jsxFileName,
        lineNumber: 193
      },
      __self: this
    }), this.renderButtons()), React.createElement(_JSXStyle, {
      id: "1959698239",
      __self: this
    }, ".calendar.jsx-1959698239{border-radius:var(--scale-alpha);box-shadow:var(--shadow-5);background-color:var(--white);}.calendar.jsx-1959698239 .CalendarMonth_table{margin-top:var(--scale-bravo);}.calendar.jsx-1959698239 .CalendarDay__default{border:none;color:var(--marine-700);font-size:1rem;}.calendar.jsx-1959698239 .CalendarDay__selected{background:var(--primary-500);color:var(--white);}.calendar.jsx-1959698239 .CalendarDay__selected_span{background:var(--primary-50);}.calendar.jsx-1959698239 .CalendarDay__hovered_span{background:var(--primary-50);}.calendar.jsx-1959698239 .CalendarMonth_caption{font-size:24px;}.calendar.jsx-1959698239 .DayPickerNavigation_button__default{border:0;}.calendar.jsx-1959698239 .CalendarDay__blocked_calendar{background:var(--transparent);color:var(--cloud-400);}.calendar.jsx-1959698239 .DayPickerNavigation{display:-webkit-box;display:-webkit-flex;display:-ms-flexbox;display:flex;-webkit-box-pack:justify;-webkit-justify-content:space-between;-ms-flex-pack:justify;justify-content:space-between;padding:0 24px;}.calendar.jsx-1959698239 .DayPickerNavigation_button{margin-top:24px;}.calendar.jsx-1959698239 .DayPickerNavigation_button__horizontalDefault{padding:0;}.calendar.jsx-1959698239 .DayPickerNavigation_button__disabled{border:0;}.calendar.jsx-1959698239 .DayPicker_weekHeader_li small{font-size:0.875rem;color:var(--cloud-900);}.calendar.jsx-1959698239 .CalendarDay__default:hover,.calendar.jsx-1959698239 .CalendarDay__default:focus{background:var(--cloud-100);}.calendar.jsx-1959698239 .CalendarDay__default:focus{outline:none;}.calendar.jsx-1959698239 .DayPickerNavigation_button:focus{outline:none;}.calendar.jsx-1959698239 .CalendarDay__blocked_calendar:hover,.calendar.jsx-1959698239 .CalendarDay__blocked_calendar:focus{background:var(--transparent);color:var(--cloud-400);}.calendar.jsx-1959698239 .CalendarDay__selected:hover,.calendar.jsx-1959698239 .CalendarDay__selected:active{background:var(--primary-500);}.calendar.jsx-1959698239 .CalendarDay__selected_span:hover,.calendar.jsx-1959698239 .CalendarDay__selected_span:active{background:var(--primary-50);}.calendar.jsx-1959698239 .CalendarDay__default.CalendarDay__selected:focus{background:var(--primary-500);color:var(--white);}.calendar.jsx-1959698239 .CalendarDay__blocked_calendar.CalendarDay__selected{background:var(--primary-500);color:var(--white);opacity:0.6;}.calendar.jsx-1959698239 .CalendarDay__blocked_calendar.CalendarDay__selected_span{background:var(--primary-50);opacity:0.6;}.calendar.jsx-1959698239 .DayPickerNavigation_button__disabled path{fill:var(--cloud-100);}"));
  }

}