import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
var _jsxFileName = "C:\\Dev\\20_gsoft\\sg-orbit\\packages\\react-components\\components\\multi-select\\src\\multi-select-dropdown.jsx";
import React from "react";
import { Dropdown, Ref } from "semantic-ui-react";
import { ITEM_SHAPE } from "./items";
import { KEYS } from "@orbit-ui/react-components-shared";
import { MultiSelectDropdownMenu } from "./multi-select-dropdown-menu";
import { MultiSelectDropdownSearchInput } from "./multi-select-dropdown-search-input";
import { MultiSelectDropdownTrigger } from "./multi-select-dropdown-trigger";
import { PureComponent, cloneElement, createRef } from "react";
import { arrayOf, bool, func, node, number, shape, string } from "prop-types";
import { debounce, isFunction, isNil } from "lodash";
export class MultiSelectDropdown extends PureComponent {
  constructor() {
    super(...arguments);

    _defineProperty(this, "state", {
      keyboardItem: null,
      keyboardIndex: null
    });

    _defineProperty(this, "_dropdownRef", createRef());

    _defineProperty(this, "handleDocumentClick", event => {
      if (this._dropdownRef.current) {
        if (!this._dropdownRef.current.contains(event.target)) {
          this.toggleVisibility(event);
        }
      }
    });

    _defineProperty(this, "handleKeyDown", event => {
      switch (event.keyCode) {
        case KEYS.esc:
          this.handleEscape(event);
          break;

        case KEYS.enter:
          this.handleEnter(event);
          break;

        case KEYS.up:
          this.handleUp(event);
          break;

        case KEYS.down:
          this.handleDown(event);
          break;
      }
    });

    _defineProperty(this, "handleEscape", event => {
      this.toggleVisibility(event);
    });

    _defineProperty(this, "handleEnter", event => {
      const {
        keyboardItem
      } = this.state;

      if (!isNil(keyboardItem)) {
        this.selectItem(event, keyboardItem);
      }
    });

    _defineProperty(this, "handleUp", () => {
      const {
        items
      } = this.props;
      const {
        keyboardIndex
      } = this.state;

      if (items.length > 0) {
        if (!isNil(keyboardIndex)) {
          if (keyboardIndex > 0) {
            const newKeyboardIndex = keyboardIndex - 1;
            this.setKeyboardItem(items[newKeyboardIndex], newKeyboardIndex);
          }
        }
      }
    });

    _defineProperty(this, "handleDown", () => {
      const {
        items
      } = this.props;
      const {
        keyboardIndex
      } = this.state;

      if (items.length > 0) {
        if (isNil(keyboardIndex)) {
          this.setKeyboardItem(items[0], 0);
        } else {
          if (keyboardIndex < items.length - 1) {
            const newKeyboardIndex = keyboardIndex + 1;
            this.setKeyboardItem(items[newKeyboardIndex], newKeyboardIndex);
          }
        }
      }
    });

    _defineProperty(this, "handleTriggerClick", event => {
      this.toggleVisibility(event);
    });

    _defineProperty(this, "handleSearchChange", (event, _ref) => {
      let {
        value
      } = _ref;
      this.onSearch(event, value, this.props);
    });

    _defineProperty(this, "handleItemClick", (event, item) => {
      this.selectItem(event, item);
    });

    _defineProperty(this, "onSearch", this.props.debounceDelay !== 0 ? debounce(this.props.onSearch, this.props.debounceDelay) : this.props.onSearch);

    _defineProperty(this, "renderTrigger", () => {
      const {
        trigger,
        triggerText,
        triggerIcon
      } = this.props;
      return cloneElement(trigger, {
        onClick: this.handleTriggerClick,
        text: triggerText,
        icon: triggerIcon
      });
    });

    _defineProperty(this, "renderSearchInput", () => {
      const {
        searchInput,
        searchIcon,
        placeholder
      } = this.props;
      return cloneElement(searchInput, {
        key: "search-input",
        onChange: this.handleSearchChange,
        icon: searchIcon,
        placeholder
      });
    });

    _defineProperty(this, "renderMenu", () => {
      const {
        menu,
        items,
        itemRenderer,
        headerRenderer,
        noResultsMessage
      } = this.props;
      const {
        keyboardItem
      } = this.state;
      return cloneElement(menu, {
        items: items,
        onItemClick: this.handleItemClick,
        itemRenderer,
        headerRenderer,
        searchInput: this.renderSearchInput(),
        noResultsMessage,
        keyboardItem
      });
    });
  }

  componentDidUpdate(prevProps) {
    const {
      items
    } = this.props;

    if (prevProps.items !== items) {
      this.setKeyboardItem(null, null);
    }
  }

  componentWillUnmount() {
    this.unbindEvents();
    this.cancelOnSearchDebounce();
  }

  cancelOnSearchDebounce() {
    if (isFunction(this.onSearch.cancel)) {
      this.onSearch.cancel();
    }
  }

  toggleVisibility(event) {
    const {
      open
    } = this.props;

    if (open) {
      this.close(event);
    } else {
      this.open(event);
    }
  }

  open(event) {
    const {
      onOpen
    } = this.props;
    this.bindEvents();
    this.setKeyboardItem(null, null);
    onOpen(event, this.props);
  }

  close(event) {
    const {
      onClose
    } = this.props;
    this.unbindEvents();
    onClose(event, this.props);
  }

  bindEvents() {
    document.addEventListener("click", this.handleDocumentClick, false);
    document.addEventListener("keydown", this.handleKeyDown, false);
  }

  unbindEvents() {
    document.removeEventListener("click", this.handleDocumentClick, false);
    document.removeEventListener("keydown", this.handleKeyDown, false);
  }

  setKeyboardItem(item, index) {
    this.setState({
      keyboardItem: item,
      keyboardIndex: index
    });
  }

  selectItem(event, item) {
    const {
      items,
      onItemSelect,
      closeOnSelect
    } = this.props;

    if (closeOnSelect || items.length === 1) {
      this.toggleVisibility(event);
    } // Defering onItemSelect ensure that document click has already been handled and the dropdown is not closed because the item is not part of the menu anymore.


    setTimeout(() => {
      const selectedItem = items.find(x => x.value === item.value);
      onItemSelect(event, selectedItem, this.props);
    }, 0);
  }

  getClasses() {
    const {
      className
    } = this.props;
    const defaultClasses = "no-icon";
    return isNil(className) ? defaultClasses : "".concat(defaultClasses, " ").concat(className);
  }

  render() {
    const {
      disabled,
      open
    } = this.props;
    return React.createElement(Ref, {
      innerRef: this._dropdownRef,
      __source: {
        fileName: _jsxFileName,
        lineNumber: 255
      },
      __self: this
    }, React.createElement(Dropdown, {
      open: open,
      trigger: this.renderTrigger(),
      className: this.getClasses(),
      disabled: disabled,
      upward: false,
      floating: true,
      __source: {
        fileName: _jsxFileName,
        lineNumber: 256
      },
      __self: this
    }, open ? this.renderMenu() : null));
  }

}

_defineProperty(MultiSelectDropdown, "defaultProps", {
  debounceDelay: 200,
  trigger: React.createElement(MultiSelectDropdownTrigger, {
    __source: {
      fileName: _jsxFileName,
      lineNumber: 37
    },
    __self: this
  }),
  menu: React.createElement(MultiSelectDropdownMenu, {
    __source: {
      fileName: _jsxFileName,
      lineNumber: 38
    },
    __self: this
  }),
  searchInput: React.createElement(MultiSelectDropdownSearchInput, {
    __source: {
      fileName: _jsxFileName,
      lineNumber: 39
    },
    __self: this
  })
});