import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
var _jsxFileName = "C:\\Dev\\20_gsoft\\sg-orbit\\packages\\react-components\\components\\multi-select\\src\\multi-select.jsx";
import React from "react";
import { AddIcon, ClearIcon, MagnifierIcon } from "@orbit-ui/icons";
import { AutoControlledPureComponent, getAutoControlledStateFromProps } from "@orbit-ui/react-components-shared";
import { Button, Dropdown, Label } from "semantic-ui-react";
import { ITEM_SHAPE } from "./items";
import { MultiSelectClearButton } from "./multi-select-clear-button";
import { MultiSelectDropdown } from "./multi-select-dropdown";
import { MultiSelectDropdownMenu } from "./multi-select-dropdown-menu";
import { MultiSelectDropdownSearchInput } from "./multi-select-dropdown-search-input";
import { MultiSelectDropdownTrigger } from "./multi-select-dropdown-trigger";
import { MultiSelectSelectedItems } from "./multi-select-selected-items";
import { arrayOf, bool, func, node, shape, string } from "prop-types";
import { cloneElement } from "react";
import { isNil } from "lodash";
import cx from "classnames";
const GROUP_ERROR_MESSAGE = "MultiSelect - When at least one item has a \"group\" property, all items must have a \"group\" property.";

function defaultItemRenderer(item, isSelected) {
  return React.createElement(Dropdown.Item, {
    text: item.text,
    value: item.value,
    selected: isSelected,
    __source: {
      fileName: _jsxFileName,
      lineNumber: 19
    },
    __self: this
  });
}

function defaultCategoryHeaderRenderer(group) {
  return React.createElement(Dropdown.Header, {
    content: group,
    __source: {
      fileName: _jsxFileName,
      lineNumber: 23
    },
    __self: this
  });
}

function defaultSelectedItemRenderer(item, _ref) {
  let {
    disabled,
    onRemove
  } = _ref;
  return React.createElement(Label, {
    basic: true,
    className: cx("large", {
      icon: !disabled
    }),
    disabled: disabled,
    __source: {
      fileName: _jsxFileName,
      lineNumber: 28
    },
    __self: this
  }, item.text, !disabled ? React.createElement(Button, {
    circular: true,
    size: "mini",
    icon: true,
    className: "transparent",
    onClick: onRemove,
    __source: {
      fileName: _jsxFileName,
      lineNumber: 31
    },
    __self: this
  }, React.createElement(ClearIcon, {
    className: "h3 w3",
    __source: {
      fileName: _jsxFileName,
      lineNumber: 32
    },
    __self: this
  })) : null);
}

export function startsWithSearch(event, items, query) {
  return items.filter(x => x.text.toUpperCase().startsWith(query.toUpperCase()));
}

function getSelectedItems(items, values) {
  // This is a mapping of the values to preserve the selection order of the items.
  return values.map(x => items.find(item => item.value === x)).filter(x => x);
}

function getAvailableItems(items, values) {
  // This is a filtering of the items to preserve the original order of the items.
  return items.filter(x => !values.includes(x.value));
}

function computeDerivedState(items, values) {
  return {
    selectedItems: getSelectedItems(items, values),
    availableItems: getAvailableItems(items, values),
    dropdownItems: getAvailableItems(items, values)
  };
}

export class MultiSelect extends AutoControlledPureComponent {
  constructor() {
    super(...arguments);

    _defineProperty(this, "state", {
      values: [],
      selectedItems: [],
      availableItems: [],
      dropdownItems: [],
      open: false
    });

    _defineProperty(this, "handleSearch", (event, query) => {
      const {
        onSearch
      } = this.props;
      const {
        availableItems
      } = this.state;
      const results = onSearch(event, availableItems, query, this.props);
      this.setState({
        dropdownItems: results
      });
    });

    _defineProperty(this, "handleItemSelect", (event, item) => {
      const {
        values
      } = this.state;
      this.setValues(event, [...values, item.value]);
    });

    _defineProperty(this, "handleRemoveSelectedItem", (event, item) => {
      const {
        values
      } = this.state;
      this.setValues(event, values.filter(x => x !== item.value));
    });

    _defineProperty(this, "handleOpen", event => {
      const {
        onVisiblityChange
      } = this.props;
      this.trySetAutoControlledStateValue({
        open: true
      });

      if (!isNil(onVisiblityChange)) {
        onVisiblityChange(event, true);
      }
    });

    _defineProperty(this, "handleClose", event => {
      const {
        onVisibilityChange
      } = this.props;
      const {
        availableItems
      } = this.state;
      this.setState({
        dropdownItems: availableItems
      });
      this.trySetAutoControlledStateValue({
        open: false
      });

      if (!isNil(onVisibilityChange)) {
        onVisibilityChange(event, false, this.props);
      }
    });

    _defineProperty(this, "handleClear", event => {
      const {
        items
      } = this.props;
      this.setValues(event, [], [], items, items);
    });

    _defineProperty(this, "renderDropDown", () => {
      const {
        itemRenderer,
        categoryHeaderRenderer,
        closeOnSelect,
        dropdown,
        triggerText,
        triggerIcon,
        searchIcon,
        placeholder,
        noResultsMessage,
        disabled
      } = this.props;
      const {
        dropdownItems,
        open
      } = this.state;
      return cloneElement(dropdown, {
        items: dropdownItems,
        onSearch: this.handleSearch,
        onItemSelect: this.handleItemSelect,
        onOpen: this.handleOpen,
        onClose: this.handleClose,
        itemRenderer,
        headerRenderer: categoryHeaderRenderer,
        closeOnSelect,
        triggerText,
        triggerIcon,
        searchIcon,
        placeholder,
        noResultsMessage,
        open,
        disabled
      });
    });

    _defineProperty(this, "renderSelectedItems", () => {
      const {
        selectedItemsComponent,
        selectedItemRenderer,
        disabled
      } = this.props;
      const {
        selectedItems
      } = this.state;
      return cloneElement(selectedItemsComponent, {
        items: selectedItems,
        selectedItemRenderer,
        onRemoveSelectedItem: this.handleRemoveSelectedItem,
        disabled
      });
    });

    _defineProperty(this, "renderClearButton", () => {
      const {
        clearButton,
        clearText,
        disabled
      } = this.props;
      const {
        selectedItems
      } = this.state;

      if (selectedItems.length === 0 || disabled) {
        return null;
      }

      return cloneElement(clearButton, {
        onClick: this.handleClear,
        text: clearText
      });
    });
  }

  componentDidMount() {
    this.validateGrouping();
  }

  componentDidUpdate(prevProps) {
    const {
      items
    } = this.props;

    if (prevProps.items !== items) {
      this.validateGrouping();
    }
  }

  static getDerivedStateFromProps(props, state) {
    const {
      items
    } = props;
    return getAutoControlledStateFromProps(props, state, MultiSelect.autoControlledProps, (_ref2) => {
      let {
        values
      } = _ref2;
      return computeDerivedState(items, values);
    });
  }

  setValues(event, newValues) {
    const {
      items,
      onValuesChange
    } = this.props;
    this.trySetAutoControlledStateValue({
      values: newValues
    }, () => computeDerivedState(items, newValues));
    onValuesChange(event, newValues, this.props);
  }

  validateGrouping() {
    const {
      items
    } = this.props;
    let isGrouped = false;
    items.forEach((x, index) => {
      if (isGrouped) {
        if (isNil(x.group)) {
          // Others have a group.
          throw new Error(GROUP_ERROR_MESSAGE);
        }
      } else {
        if (!isNil(x.group)) {
          if (index === 0) {
            isGrouped = true;
          } else {
            // This item have a group but others dont.
            throw new Error(GROUP_ERROR_MESSAGE);
          }
        }
      }
    });
  }

  render() {
    const {
      className
    } = this.props;
    const defaultClasses = "flex flex-wrap";
    const classes = isNil(className) ? defaultClasses : "".concat(defaultClasses, " ").concat(className);
    return React.createElement("div", {
      className: classes,
      __source: {
        fileName: _jsxFileName,
        lineNumber: 279
      },
      __self: this
    }, this.renderDropDown(), this.renderSelectedItems(), React.createElement("div", {
      className: "mr2 mb2",
      __source: {
        fileName: _jsxFileName,
        lineNumber: 282
      },
      __self: this
    }, this.renderClearButton()));
  }

}

_defineProperty(MultiSelect, "propTypes", {
  items: arrayOf(shape(ITEM_SHAPE)).isRequired,
  values: arrayOf(string),
  defaultValues: arrayOf(string),
  onValuesChange: func.isRequired,
  onSearch: func,
  onVisibilityChange: func,
  dropdown: node,
  itemRenderer: func,
  categoryHeaderRenderer: func,
  closeOnSelect: bool,
  noResultsMessage: string,
  triggerText: string,
  triggerIcon: node,
  searchIcon: node,
  placeholder: string,
  selectedItemsComponent: node,
  selectedItemRenderer: func,
  clearButton: node,
  clearText: string,
  defaultOpen: bool,
  open: bool,
  disabled: bool,
  className: string
});

_defineProperty(MultiSelect, "defaultProps", {
  dropdown: React.createElement(MultiSelectDropdown, {
    __source: {
      fileName: _jsxFileName,
      lineNumber: 89
    },
    __self: this
  }),
  onSearch: startsWithSearch,
  itemRenderer: defaultItemRenderer,
  categoryHeaderRenderer: defaultCategoryHeaderRenderer,
  closeOnSelect: false,
  noResultsMessage: "No results",
  triggerText: "Add",
  triggerIcon: React.createElement(AddIcon, {
    className: "w3 h3 fill-marine-700 ml2",
    __source: {
      fileName: _jsxFileName,
      lineNumber: 96
    },
    __self: this
  }),
  searchIcon: React.createElement(MagnifierIcon, {
    className: "w4 h4 fill-marine-500",
    __source: {
      fileName: _jsxFileName,
      lineNumber: 97
    },
    __self: this
  }),
  placeholder: "Search",
  selectedItemsComponent: React.createElement(MultiSelectSelectedItems, {
    __source: {
      fileName: _jsxFileName,
      lineNumber: 99
    },
    __self: this
  }),
  selectedItemRenderer: defaultSelectedItemRenderer,
  clearButton: React.createElement(MultiSelectClearButton, {
    __source: {
      fileName: _jsxFileName,
      lineNumber: 101
    },
    __self: this
  }),
  clearText: "Clear all",
  disabled: false
});

_defineProperty(MultiSelect, "autoControlledProps", ["values", "open"]);

_defineProperty(MultiSelect, "Dropdown", MultiSelectDropdown);

_defineProperty(MultiSelect, "Trigger", MultiSelectDropdownTrigger);

_defineProperty(MultiSelect, "Menu", MultiSelectDropdownMenu);

_defineProperty(MultiSelect, "SearchInput", MultiSelectDropdownSearchInput);

_defineProperty(MultiSelect, "SelectedItems", MultiSelectSelectedItems);

_defineProperty(MultiSelect, "ClearButton", MultiSelectClearButton);