import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
var _jsxFileName = "C:\\Dev\\20_gsoft\\sg-orbit\\packages\\react-components\\components\\search-input\\src\\remote-search-input.jsx";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

import React from "react";
import { AutoControlledPureComponent, InvalidOperationError, KEYS, cancellablePromise, defer, ensure, getAutoControlledStateFromProps, httpGet } from "@orbit-ui/react-components-shared";
import { SearchInputController } from "./search-input-controller";
import { bool, func, number, string } from "prop-types";
import { debounce, isArray, isNil } from "lodash";

function defaultResultsFetcher(event, url, data, options) {
  return new Promise((resolve, reject) => {
    httpGet(_objectSpread({
      url,
      data
    }, options)).then(response => {
      if (response.ok) {
        resolve(response.json());
      } else {
        reject(new InvalidOperationError("Remote Search Input - The request failed with response: \"".concat(response.statusText, "\"")));
      }
    }).catch(error => {
      if (error.isTimeout) {
        reject(new InvalidOperationError("Remote Search Input - The request timed out."));
      } else {
        reject(error);
      }
    });
  });
}
/**
 * Create an instance of the default results fetcher
 * @param {string} url - The query url.
 * @param {string} [queryParameter="query"] - The query parameter.
 * @param {Object} [options]
 * @param {Object} [options.queryData] - Additional query data.
 * @param {Object} [options.requestOptions]
 * @param {number} [options.requestOptions.timeout] - Query timeout value in milliseconds.
 * @param {Object} [options.requestOptions.*] - Any fetch API options.
 * @returns {Function} - The fetcher instance
 */


export function useDefaultResultsFetcher(url) {
  let queryParameter = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "query";
  let {
    queryData = {},
    requestOptions
  } = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  ensure(url, "url", "useDefaultResultsFetcher").isNotNullOrEmpty();
  return (event, query) => {
    const data = _objectSpread({
      [queryParameter]: query
    }, queryData);

    return defaultResultsFetcher(event, url, data, requestOptions);
  };
}

function isPromise(value) {
  return !isNil(value) && !isNil(value.then);
}

export class RemoteSearchInput extends AutoControlledPureComponent {
  constructor() {
    super(...arguments);

    _defineProperty(this, "state", {
      open: false,
      isLoading: false,
      results: []
    });

    _defineProperty(this, "_fetchResultsPromise", null);

    _defineProperty(this, "handleSearch", (event, query) => {
      this.cancelFetch();
      this.search(event, query);
    });

    _defineProperty(this, "handleValueChange", (event, value) => {
      const {
        onValueChange
      } = this.props;
      this.close(event);

      if (isNil(value)) {
        this.handleClear(event);
      }

      onValueChange(event, value, this.props);
    });

    _defineProperty(this, "handleClear", () => {
      this.cancelFetch();
      this.hideLoading();
    });

    _defineProperty(this, "handleBlur", event => {
      const {
        onBlur
      } = this.props;
      this.cancelFetch();
      this.hideLoading();
      this.close(event);

      if (!isNil(onBlur)) {
        onBlur(event, this.props);
      }
    });

    _defineProperty(this, "handleKeyDown", event => {
      const {
        onKeyDown
      } = this.props;

      if (event.keyCode === KEYS.esc) {
        this.close(event);
      }

      if (!isNil(onKeyDown)) {
        onKeyDown(event, this.props);
      }
    });

    _defineProperty(this, "search", debounce(async (event, query) => {
      const {
        onResults,
        minCharacters
      } = this.props;

      if (query.length >= minCharacters) {
        this.showLoading();

        try {
          let results = await this.fetch(event, query);

          if (!isNil(onResults)) {
            results = onResults(results, query, this.props);

            if (!isArray(results)) {
              throw new InvalidOperationError("Remote Search Input - onResults expect a return value of type array.");
            }
          }

          this.hideLoading();
          this.open(event);
          this.setState({
            results: results
          });
        } catch (error) {
          // To cancel a promise it must be rejected, ignore it. If it's something else, bubble up.
          if (error.isCancelled !== true) {
            throw error;
          }
        }
      } else {
        this.close(event);
        this.setState({
          results: []
        });
      }
    }, this.props.debounceDelay, {
      leading: true
    }));

    _defineProperty(this, "showLoading", defer(() => {
      this.setState({
        isLoading: true
      });
    }, this.props.loadingDelay));
  }

  static getDerivedStateFromProps(props, state) {
    return getAutoControlledStateFromProps(props, state, RemoteSearchInput.autoControlledProps);
  }

  componentWillUnmount() {
    this.cancelFetch();
    this.cancelLoading();
    this.search.cancel();
  }

  fetch(event, query) {
    const {
      onFetchResults
    } = this.props;
    const promise = onFetchResults(event, query, this.props);

    if (!isPromise(promise)) {
      throw new InvalidOperationError("RemoteSearchInput - onFetchResults expect a return value of type Promise.");
    }

    this._fetchResultsPromise = cancellablePromise(promise);
    return this._fetchResultsPromise.promise;
  }

  cancelFetch() {
    if (!isNil(this._fetchResultsPromise)) {
      this._fetchResultsPromise.cancel();
    }
  }

  open(event) {
    const {
      onVisibilityChange
    } = this.props;
    this.trySetAutoControlledStateValue({
      open: true
    });

    if (!isNil(onVisibilityChange)) {
      onVisibilityChange(event, true);
    }
  }

  close(event) {
    const {
      onVisibilityChange
    } = this.props;
    this.trySetAutoControlledStateValue({
      open: false
    });

    if (!isNil(onVisibilityChange)) {
      onVisibilityChange(event, false);
    }
  }

  hideLoading() {
    const {
      isLoading
    } = this.state;
    this.cancelLoading();

    if (isLoading) {
      this.setState({
        isLoading: false
      });
    }
  }

  cancelLoading() {
    this.showLoading.cancel();
  }

  render() {
    const {
      value,
      defaultValue,
      resultRenderer,
      clearOnSelect,
      noResultsMessage,
      minCharacters,
      placeholder,
      disabled,
      autofocus,
      className
    } = this.props;
    const {
      open,
      isLoading,
      results
    } = this.state;
    return React.createElement(SearchInputController, {
      open: open,
      results: results,
      value: value,
      defaultValue: defaultValue,
      onValueChange: this.handleValueChange,
      onSearch: this.handleSearch,
      onBlur: this.handleBlur,
      onKeyDown: this.handleKeyDown,
      resultRenderer: resultRenderer,
      clearOnSelect: clearOnSelect,
      noResultsMessage: noResultsMessage,
      minCharacters: minCharacters,
      debounceDelay: 0,
      placeholder: placeholder,
      disabled: disabled,
      loading: isLoading,
      autofocus: autofocus,
      className: className,
      __source: {
        fileName: _jsxFileName,
        lineNumber: 253
      },
      __self: this
    });
  }

}

_defineProperty(RemoteSearchInput, "propTypes", {
  value: string,
  defaultValue: string,
  onValueChange: func.isRequired,
  onFetchResults: func.isRequired,
  onResults: func,
  onBlur: func,
  onKeyDown: func,
  onVisibilityChange: func,
  resultRenderer: func,
  clearOnSelect: bool,
  noResultsMessage: string,
  debounceDelay: number,
  loadingDelay: number,
  minCharacters: number,
  placeholder: string,
  defaultOpen: bool,
  open: bool,
  disabled: bool,
  autofocus: bool,
  className: string
});

_defineProperty(RemoteSearchInput, "defaultProps", {
  loadingDelay: 150,
  minCharacters: 1,
  debounceDelay: 200,
  autofocus: false
});

_defineProperty(RemoteSearchInput, "autoControlledProps", ["open"]);